package com.supwisdom.dlpay.api.bean

import com.supwisdom.dlpay.exception.RequestParamCheckException
import com.supwisdom.dlpay.framework.util.DateUtil
import com.supwisdom.dlpay.framework.util.MD5
import com.supwisdom.dlpay.framework.util.StringUtil
import com.supwisdom.dlpay.framework.util.TradeErrorCode
import com.supwisdom.dlpay.util.ConstantUtil

open class APIRequestParam {
    open val param_map = mutableMapOf<String, String?>()

    open fun checkSign(key: String): Boolean {
        val sign = param_map["sign"]
        val signType = param_map["sign_type"] ?: "MD5"
        if (StringUtil.isEmpty(sign)) return false //未签名

        //判断签名
        val signdata = StringUtil.createLinkString(StringUtil.paraFilter(param_map))
        val md5Sign = MD5.encodeByMD5(signdata + key) //默认MD5
        if (sign.equals(md5Sign, true)) {
            return true
        }
        return false
    }
}


// ============================ USER ============================ //
class OpenUserParam : APIRequestParam() {
    var uid: String = ""    //第三方用户ID,必传
    var name: String = ""   //必传
    var sex: String? = null
    var idtype: String = "" //必传
    var idno: String = ""   //必传
    var mobile: String? = null
    var tel: String? = null
    var email: String? = null
    var address: String? = null
    var zipcode: String? = null
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(uid)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "用户唯一号不能为空")
        if (StringUtil.isEmpty(name)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "姓名不能为空")
        if (!StringUtil.isEmpty(sex) && ConstantUtil.SEX_MALE != sex && ConstantUtil.SEX_FEMALE != sex) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "性别非法[male|female]")
        if (StringUtil.isEmpty(idtype) || !ConstantUtil.IDTYPE_DICTS.contains(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件类型非法")
        if (StringUtil.isEmpty(idno) || !StringUtil.isCharAndNum(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空，且仅支持数字和字母")
        if (ConstantUtil.IDTYPE_IDENTITY == idtype && !StringUtil.isIdentity(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "身份证格式错误")
        if (!StringUtil.isEmpty(mobile) && !StringUtil.isMobile(mobile)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "手机号格式错误")
        if (!StringUtil.isEmpty(email) && !StringUtil.isEmail(email)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "邮箱格式错误")

        param_map.plus(mapOf("uid" to uid, "name" to name, "sex" to sex, "idtype" to idtype, "idno" to idno, "mobile" to mobile, "tel" to tel, "email" to email, "address" to address, "zipcode" to zipcode, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}

class QueryUserParam : APIRequestParam() {
    var userid: String? = null  // 用户ID二选一 (两者都传取userid)
    var uid: String? = null    // 用户ID二选一
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(userid) && StringUtil.isEmpty(uid)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "用户唯一号不能为空")

        param_map.plus(mapOf("userid" to userid, "uid" to uid, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}

class ModifyUserParam : APIRequestParam() {
    var userid: String? = null  // 用户ID二选一 (两者都传取userid)
    var uid: String? = null    // 用户ID二选一
    var name: String? = null
    var sex: String? = null
    var idtype: String? = null
    var idno: String? = null
    var mobile: String? = null
    var tel: String? = null
    var email: String? = null
    var address: String? = null
    var zipcode: String? = null
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(userid) && StringUtil.isEmpty(uid)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "用户唯一号不能为空")
        if (!StringUtil.isEmpty(sex) && ConstantUtil.SEX_MALE != sex && ConstantUtil.SEX_FEMALE != sex) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "性别非法[male|female]")
        if (!StringUtil.isEmpty(idtype) && !ConstantUtil.IDTYPE_DICTS.contains(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件类型非法")
        if (!StringUtil.isEmpty(idno) && !StringUtil.isCharAndNum(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空，且仅支持数字和字母")
        if (!StringUtil.isEmpty(idno) && StringUtil.isEmpty(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空时，请指定证件类型")
        if (!StringUtil.isEmpty(idno) && ConstantUtil.IDTYPE_IDENTITY == idtype && !StringUtil.isIdentity(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "身份证格式错误")
        if (!StringUtil.isEmpty(mobile) && !StringUtil.isMobile(mobile)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "手机号格式错误")
        if (!StringUtil.isEmpty(email) && !StringUtil.isEmail(email)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "邮箱格式错误")


        param_map.plus(mapOf("userid" to userid, "uid" to uid, "name" to name, "sex" to sex, "idtype" to idtype, "idno" to idno, "mobile" to mobile, "tel" to tel, "email" to email, "address" to address, "zipcode" to zipcode, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}


// ============================ SHOP ============================ //
class OpenShopParam : APIRequestParam() {
    var shopUniqueId: String = "" //必传
    var shoptype: Int = 1
    var fshopid: Int? = null
    var shopname: String = "" //必传
    var contactman: String? = null
    var idtype: String? = null
    var idno: String? = null
    var mobile: String? = null
    var email: String? = null
    var address: String? = null
    var zipcode: String? = null
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(shopUniqueId)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "商户唯一号不能为空")
        if (ConstantUtil.SHOPTYPE_GROUP != shoptype && ConstantUtil.SHOPTYPE_LEAF != shoptype) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "请指定商户类型")
        if (StringUtil.isEmpty(shopname)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "商户名称不能为空")
        if (!StringUtil.isEmpty(idtype) && !ConstantUtil.IDTYPE_DICTS.contains(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件类型非法")
        if (!StringUtil.isEmpty(idno) && !StringUtil.isCharAndNum(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空，且仅支持数字和字母")
        if (!StringUtil.isEmpty(idno) && StringUtil.isEmpty(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空时，请指定证件类型")
        if (!StringUtil.isEmpty(mobile) && !StringUtil.isMobile(mobile)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "手机号格式错误")
        if (!StringUtil.isEmpty(email) && !StringUtil.isEmail(email)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "邮箱格式错误")

        param_map.plus(mapOf("shopUniqueId" to shopUniqueId, "shoptype" to shoptype, "fshopid" to fshopid, "shopname" to shopname, "contactman" to contactman, "idtype" to idtype, "idno" to idno, "mobile" to mobile, "email" to email, "address" to address, "zipcode" to zipcode, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}

class QueryShopParam : APIRequestParam() {
    var shopid: Int? = null  //注册时返回的shopid
    var shopaccno: String? = null  //注册时返回的shopaccno
    var shopUniqueId: String? = null //注册传的商户唯一号
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(shopUniqueId) && null == shopid && StringUtil.isEmpty(shopaccno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "商户唯一标志不能为空")

        param_map.plus(mapOf("shopUniqueId" to shopUniqueId, "shopid" to shopid, "shopaccno" to shopaccno, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}


// ============================ CONSUME ============================ //
class ConsumeFeetype {
    var feetype: String = ""
    var amount: Int = 0
}

class QueryDtlResultParam : APIRequestParam() {
    var refno:String?=null //二选一
    var billno:String?=null //二选一 (billno+shopaccno） 传billno时，shopaccno必传
    var shopaccno: String?=null
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(refno) && (StringUtil.isEmpty(billno) || StringUtil.isEmpty(shopaccno))) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "流水唯一号不能为空")

        param_map.plus(mapOf("refno" to refno, "billno" to billno, "shopaccno" to shopaccno,  "sign" to sign, "sign_type" to sign_type))
        return true
    }
}

class CitizenCardPayinitParam : APIRequestParam() {
    var cardNo: String = "" //必传
    var shopaccno: String = "" //必传
    var amount: Int = 0 //必传
    var feelist: List<ConsumeFeetype>? = null// 的json字符串
    var billno: String = "" //必传
    var transdate: String = "" //必传
    var transtime: String = "" //必传
    var extend_data: String? = null //扩展参数，原样返回
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(cardNo)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "卡唯一号不能为空")
        if (StringUtil.isEmpty(shopaccno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "请指定交易商户")
        if (amount <= 0) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "交易金额必须大于零")
        if (StringUtil.isEmpty(billno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "对接系统唯一订单号不能为空")
        if (!DateUtil.checkDatetimeValid(transdate, DateUtil.DATE_FMT)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "交易日期错误[yyyyMMdd]")
        if (!DateUtil.checkDatetimeValid(transtime, DateUtil.TIME_FMT)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "交易时间错误[HHmmss]")

        param_map.plus(mapOf("cardNo" to cardNo, "amount" to amount, "feelist" to feelist, "shopaccno" to shopaccno, "billno" to billno, "transdate" to transdate, "transtime" to transtime, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}

class CitizenCardPayfinishParam : APIRequestParam() {
    var refno:String=""
    var sign: String = "" //必传
    var sign_type: String? = null

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(refno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "交易参考号不能为空")

        param_map.plus(mapOf("refno" to refno, "sign" to sign, "sign_type" to sign_type))
        return true
    }
}




